
/* JavaScript content from js/app/challengeHandler.js in folder common */
/*global define,WL*/

define('app/challengeHandler', [
  'dojo/_base/lang',
  "app/nativeMethods"],
  function (lang, nativeMethods) {

    var challengeHandlerMixin = {
      isCustomResponse: function (response) {
        if (!response || !response.responseJSON || response.responseText === null) {
          return false;
        }
        if (response.responseJSON.authStatus !== 'undefined') {
          return true;
        }
        return false;
      },
      handleChallenge: function (response) {
        var authRequired = response.responseJSON.authStatus;

        if (authRequired === "required") {
          var error = response.responseJSON.errorMessage;

          if (error) {
            console.error('LOGIN ERROR', error);
          } else {
            nativeMethods.exec(console.info.bind(console), console.error, 'Login', 'loginRequired', []);
          }
        } else if (authRequired === "complete") {
          nativeMethods.exec(console.info.bind(console), console.error, 'Login', 'loginDidSucceed', []);

          if (this.activeRequest) {
            // This means the challenge handler was invoked indirectly (i.e. adapter call),
            // rather than with an explicit login.

            this.submitSuccess();

            // FIXME:
            // If the user explicitly cancels login on the login page, need to do:
            // * challengeHandler.onUnresponsiveHost() (may be a WL5-only bug), then
            // * challengeHandler.submitFailure()
          }
        }
      },
      handleFailure: function (failure) {
        var error = new Error(failure);
        error.failure = failure;
        throw error;
      }
    };

    var challengeHandler = WL.Client.createChallengeHandler("WCSAuthenticationRealm");

    lang.mixin(challengeHandler, challengeHandlerMixin);

    return challengeHandler;
  });

function doExplicitLogin() {
  WL.Client.login("WCSAuthenticationRealm");
}