
/* JavaScript content from js/app/models/Product.js in folder common */
/*global define*/

define("app/models/Product", [
  "dojo/Stateful",
  "dojo/_base/declare",
  "dojo/_base/array"],
  function (Stateful, declare, array) {
    var Product = declare("app/models/Product", [Stateful], {
      constructor: function (opts) {
        this.wlProduct = opts.wlProduct;
      },
      hasAverageRating: function () {
        return !!(this.averageRating && this.averageRating.length);
      },
      hasEssentialExtras: function () {
        return !!(this.essentialExtras && this.essentialExtras.length);
      },
      hasAlsoInRange: function () {
        return !!(this.alsoInRange && this.alsoInRange.length);
      },
      hasAlternatives: function () {
        return !!(this.alternatives && this.alternatives.length);
      },
      hasGuides: function () {
        return !!(this.guides);
      },
      hasCurrentPrice: function () {
        return !!(this.currentPrice);
      },
      hasWasPrice: function () {
        return !!(this.wasPrice && this.wasPrice !== "0.00");
      },
      isOnlineExclusive: function () {
        return !!(this.isOnlineExclusive && this.isOnlineExclusive === true);
      },
      hasSavingsMessage: function () {
        return !!(this.savingsMessage && typeof this.savingsMessage[0].value === "string");
      },
      hasNectarPoints: function () {
        return !!(this.nectarPoints);
      },
      hasDescription: function () {
        return !!(this.description);
      },
      hasReturnsPolicy: function () {
        return !!(this.returnsPolicy);
      },
      hasBulletinText: function() {
        return !!(this.bulletinText);
      }
    });

    function extractRelatedProducts(wlProduct) {

      var opts = wlProduct.RelatedProducts.Product;

      var essentialExtras = (array.filter(opts, function (opt) {
        return opt.type === 'essentialExtra';
      }));

      var alternatives = (array.filter(opts, function (opt) {
        return opt.type === 'alternative' || opt.type === 'replacement';
      }));

      var alsoInRange = (array.filter(opts, function (opt) {
        return opt.type === 'nonEssentialExtra';
      }));

      return {
        essentialExtras : essentialExtras,
        alternatives : alternatives,
        alsoInRange: alsoInRange
      };
    }

    function extractRatingInformation(wlProduct) {
      var opts = wlProduct.ExtendedAttributes.Attribute;

      var averageRating = (array.filter(opts, function (opt) {
        return opt.Name === 'Rating';
      })[0] || {value: ""}).Value;

      var numberOfRatings = (array.filter(opts, function (opt) {
        return opt.Name === 'NumberOfRatings';
      })[0] || {value: ""}).Value;

      return {
        averageRating: averageRating,
        numberOfRatings: numberOfRatings
      };
    }

    function extractPurchasingOptions(wlProduct) {
      var purchasingOptions = wlProduct.PurchasingOptions,
        optionIndex = 0,
        commentaryIndex = 0,
        isOnlineExclusive = false,
        inStoreReservable = false,
        homeDelivery = false,
        inStockForHomeDelivery = false,
        inStore = false,
        prePayOptionAvailable = false,
        inStoreOrderable = false,
        maxQuantity = 99,
        reserveText = "",
        deliveryLeadTimeText = "",
        deliveryLeadTimeText2 = "";

      if (purchasingOptions.onlineExclusive === true) {
        isOnlineExclusive = true;
      }

      if (purchasingOptions.Option) {

        for (optionIndex; optionIndex < purchasingOptions.Option.length; optionIndex++) {

          if (purchasingOptions.Option[optionIndex].type === "inStoreReservable") {
            inStoreReservable = true;
          }

          if (purchasingOptions.Option[optionIndex].type === "homeDelivery") {
            homeDelivery = true;
            if (purchasingOptions.Option[optionIndex].inStock) {
              inStockForHomeDelivery = true;
            }
          }

          if (purchasingOptions.Option[optionIndex].type === "inStore") {
            inStore = true;
          }

          if (purchasingOptions.Option[optionIndex].type === "prePayOptionAvailable") {
            prePayOptionAvailable = true;
          }

          if (purchasingOptions.Option[optionIndex].type === "inStoreOrderable") {
            inStoreOrderable = true;
          }

          if (typeof purchasingOptions.Option[optionIndex].MaximumQuantity === 'number') {
            maxQuantity = purchasingOptions.Option[optionIndex].MaximumQuantity;
          }

          for (commentaryIndex = 0; commentaryIndex < purchasingOptions.Option[optionIndex].Commentary.length; commentaryIndex++) {

            if (purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].type === "reserveText" && typeof purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value === "string") {
              reserveText = purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value;
            }

            if (purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].type === "deliveryLeadTimeText" && commentaryIndex === 0 && typeof purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value === "string") {
              deliveryLeadTimeText = purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value;
            }

            if (purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].type === "deliveryLeadTimeText" && commentaryIndex === 1 && typeof purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value === "string") {
              deliveryLeadTimeText2 = purchasingOptions.Option[optionIndex].Commentary[commentaryIndex].value;
            }
          }
        }

      }

      var canCheckStock = false,
        storeOptions = [],
        fulfilmentOptions = Product.fulfilmentOptions,
        deliveryOptions = [],
        currentlyOutOfStock = false,
        addToBasket = false,
        deliveryMessages = [];

      if (inStoreReservable === true) {
        canCheckStock = true;
      }

      if (inStore === true || inStoreReservable === true || inStoreOrderable === true || prePayOptionAvailable === true) {
        fulfilmentOptions.CollectionInStore.message = reserveText;
        storeOptions.push(fulfilmentOptions.CollectionInStore);
      }

      if (homeDelivery === true && inStockForHomeDelivery === true) {
        deliveryMessages.push(deliveryLeadTimeText);
        fulfilmentOptions.HomeDelivery.message = deliveryMessages;
        if (deliveryMessages.length) {
          deliveryMessages.push(deliveryLeadTimeText2);
        }
        deliveryOptions.push(fulfilmentOptions.HomeDelivery);
      }

      if (homeDelivery === true && (inStockForHomeDelivery === false || inStockForHomeDelivery === undefined)) {
      
	      if (deliveryMessages.length == 0) {
	      	deliveryMessages.push(deliveryLeadTimeText);
	        fulfilmentOptions.NoHomeDelivery.message = deliveryMessages;
	        if (deliveryMessages.length && deliveryLeadTimeText2) {
	          deliveryMessages.push(deliveryLeadTimeText2);
	        }
	        deliveryOptions.push(fulfilmentOptions.NoHomeDelivery);
	      } else { 
	        // This code in this else is the previous version
	        // It will always fail
	        var p;
	        for (p in deliveryMessages) {
	          if (deliveryMessages.hasOwnProperty(p)) {
	            deliveryMessages.push(deliveryLeadTimeText);
	            fulfilmentOptions.HomeDelivery.message = deliveryMessages;
	            if (deliveryMessages.length) {
	              deliveryOptions.push(deliveryLeadTimeText2);
	            }
	            deliveryOptions.push(fulfilmentOptions.NoHomeDelivery);
	          }
	        }
	      }
      }

      if (!(inStore === true || inStoreReservable === true || inStoreOrderable === true || prePayOptionAvailable === true || homeDelivery === true)) {
        currentlyOutOfStock = true;
      }

      if (inStoreReservable === true || prePayOptionAvailable === true || (homeDelivery === true && inStockForHomeDelivery == true)) {
        addToBasket = true;
      }

      return {
        deliveryOptions: deliveryOptions,
        storeOptions: storeOptions,
        canCheckStock: canCheckStock,
        currentlyOutOfStock: currentlyOutOfStock,
        addToBasket: addToBasket,
        isOnlineExclusive: isOnlineExclusive,
        maxQuantity: maxQuantity
      };

    }

    Product.fulfilmentOptions = {
      CollectionInStore: {
        option: "tick",
        icon: "store"
      },
      HomeDelivery: {
        option: "tick",
        icon: "truck"
      },
      NoHomeDelivery: {
        option: "cross",
        icon: "truck"
      }
    };

    function extractPrice(wlProduct) {

      if (wlProduct.PricingInformation) {
        var opts = wlProduct.PricingInformation.Price;

        var current = (array.filter(opts, function (opt) {
          return opt.type === 'current';
        })[0] || {value: ""}).Amount.value;

        if (current.toString().indexOf(".") === -1) {
          current += ".00";
        }

        current = (current*1).toFixed(2);

        var was = (array.filter(opts, function (opt) {
          return opt.type === 'was';
        })[0] || {value: ""}).Amount.value;

        if (was.toString().indexOf(".") === -1) {
          was += ".00";
        }

        var savingsMessage = (array.filter(opts, function (opt) {
          return opt.type === 'current';
        })[0] || {value: ""}).Commentary;

        var nectarPoints = wlProduct.PricingInformation.LoyaltyValue[0].value;

        return {
          current: current,
          was: was,
          savingsMessage: savingsMessage,
          nectarPoints: nectarPoints
        };
      } else {
        return false;
      }
    }

    Product.fromWorklightPayload = function fromWorklightPayload(wlPayload) {
      var wlProduct = wlPayload.ProductList.Product[0];
      var wlResponse = wlPayload;
      var product = new Product({wlProduct: wlProduct, wlResponse: wlResponse});

      var relatedProducts = extractRelatedProducts(wlProduct);
      var price = extractPrice(wlProduct);
      var rating = extractRatingInformation(wlProduct);
      var purchasingOptions = extractPurchasingOptions(wlProduct);

      product.set({
        id: wlProduct.id,
        name: wlProduct.DescriptionList.Description.name,
        description: wlProduct.DescriptionList.Description.longHtml,
        nectarPoints: price.nectarPoints,
        currentPrice: price.current,
        wasPrice: price.was,
        savingsMessage: price.savingsMessage,
        essentialExtras: relatedProducts.essentialExtras,
        alternatives: relatedProducts.alternatives,
        alsoInRange: relatedProducts.alsoInRange,
        images: wlProduct.AssociatedMedia.Content,
        isOnlineExclusive: purchasingOptions.isOnlineExclusive,
        averageRating: rating.averageRating,
        numberOfRatings : rating.numberOfRatings,
        deliveryOptions: purchasingOptions.deliveryOptions,
        storeOptions: purchasingOptions.storeOptions,
        buffer: purchasingOptions.buffer,
        canCheckStock: purchasingOptions.canCheckStock,
        currentlyOutOfStock: purchasingOptions.currentlyOutOfStock,
        addToBasket: purchasingOptions.addToBasket,
        returnsPolicy: wlProduct.ReturnsPolicy,
        bulletinText: wlProduct.Bulletins,
        maximumQuantity: purchasingOptions.maxQuantity
      });

      return product;
    };

    return Product;
  });