
/* JavaScript content from js/app/models/Refinement.js in folder common */
/*global define*/

define("app/models/Refinement", [
  "dojo/_base/lang",
  "dojo/_base/array",
  "dojo/_base/declare",
  "dojo/Stateful",
  "app/models/selectionDescription",
  "app/models/Selection",
  "app/models/refinementsHelper"],
  function (lang, array, declare, Stateful, selectionDescription, selection, refinementsHelper) {
    var Refinement = declare("app/models/Refinement", [Stateful], {
      constructor: function (args) {
        args.refinements = args.refinements || [];
        args.showExtraRefinements = args.showExtraRefinements;
        var refinements = args.refinements;
        var showExtraRefinements = args.showExtraRefinements;
        this.refinements = refinements;
        this.showExtraRefinements = showExtraRefinements;
        this.parent = args.parent;
        this.name = args.name;

        // convert the child refinements from data objects to model instances
        // use a for loop and inplace swap because args are assigned to the
        // instance again by dojo after the ctor
        for (var i = 0; i < refinements.length; i++) {
          var dataObject = refinements[i];

          // this workaround stashes maintains the list of L1 categories regardless
          // of search response
          if (Refinement.__stashedCategories__ && dataObject.name === 'Categories') {
            var categories = Refinement.__stashedCategories__;
            categories.parent = this;
            Refinement.register(categories);
            refinements[i] = categories;
          } else {
            var childArgs = lang.mixin({}, dataObject, { parent: this });
            refinements[i] = new Refinement(childArgs);
          }
        }

        Refinement.register(this);
      },
      isRoot: function () {
        return this.parent === undefined;
      },
      isSelected: function () {
        return selection.isSelected(this);
      },
      toggleSelection: function () {
        if (this.isSelected()) {
          this.deselect();
        } else {
          this.select();
        }
      },
      deselect: function () {
        return selection.deselect(this);
      },
      select: function () {
        if (!this.parent.multiSelect) {
          this.parent.refinements.forEach(function (sibling) {
            if (sibling !== this) {
              selection.deselect(sibling);
            }
          }.bind(this));
        }
        return selection.select(this);
      },
      id: function () {
        if (!this.__id__) {
          if (this.parent) {
            this.__id__ = this.path().join(',');
          }
        }

        return this.__id__;
      },
      path: function () {
        if (this.parent) {
          return this.parent.path().concat([this.name]);
        }

        return [];
      },
      hasSubSelection: function () {
        if (this.name === 'Price') {
          return selection.price && (!isNaN(selection.price.min) || !isNaN(selection.price.max));
        }

        return (!!this.selectedSubRefinements().length) || array.some(this.refinements, function (refinement) {
          return refinement.hasSubSelection();
        });
      },
      categoriesRefinement: function () {
        return this.refinements.filter(function (r) {
          return r.isCategories();
        })[0];
      },
      isCategories: function () {
        return this.title === 'Categories';
      },
      selectionDescription: function () {
        if (this.hasSubSelection()) {
          var desc = selectionDescription[this.name] || selectionDescription.All;
          return desc.call(this);
        }
        return "All";
      },
      selectedSubRefinements: function () {
        return this.refinements.filter(function (r) {
          return r.isSelected();
        });
      },
      isLeaf: function () {
        return this.refinements.length === 0;
      },
      clear: function () {
        this.deselect();
        this.refinements.forEach(function (child) {
          child.clear();
        });
        if (selection.price) {
          selection.deselectPrice(this);
        }
      }
    });

    Refinement.register = function (refinement) {
      this.byId[refinement.id()] = refinement;
    };
    Refinement.typeConstructorLookup = {};
    Refinement.typeConstructor = function (type) {
      return this.typeConstructorLookup[type] || Refinement;
    };

    Refinement.build = function (refinementsList, options) {
      options = options || {};
      Refinement.byId = {};

      var refinement = new Refinement({
        name: "Refine",
        title: "Refine",
        subview: "Refine",
        refinements: refinementsHelper.normalizeRefinements(refinementsList),
        showExtraRefinements: options.showExtraRefinements
      });

      if (options.stashCategories) {
        this.__stashedCategories__ = refinement.categoriesRefinement();
      }

      return refinement;
    };

    return Refinement;
  });