
/* JavaScript content from js/app/models/refinementsHelper.js in folder common */
/*global define*/

define("app/models/refinementsHelper",
  function () {
    var refinementsHelper = {

      findOrCreateRefinement: function (name, refinements) {

        switch (name) {
        case 'Categories':
          return this.findRefinement('ParentCatalogGroup', name, refinements);
        case 'Brand':
          return this.findRefinement('ManufacturerName', name, refinements);
        case 'Price':
          return this.refinementsTemplate()['Price'];
        case 'Rating':
          return this.refinementsTemplate()['Customer Rating'];
        default:
          return this.findRefinement(name, name, refinements);
        }

      },

      findRefinement: function (dataName, uiName, refinements) {
        var r;

        for (var i = 0; i < refinements.length; i++) {
          r = refinements[i];
          if (dataName === r.Name) {
            return this.reshape(r, uiName);
          }
        }
      },

      reshape: function (refinement, uiName) {
        var template = this.refinementsTemplate(refinement)[uiName];

        template = template || this.refinementsTemplate(refinement, uiName);
        var refinements = template.refinements;

        var options = refinement.OptionList.Option.map(function (opt) {
          return {
            name: opt.Name,
            widget: "TextWithCount",
            count: opt.count,
            parameter: opt.Parameter,
            multiSelect: refinement.multiSelect
          };
        }.bind(this));

        Array.prototype.push.apply(refinements, options);

        return template;
      },

      normalizeRefinements: function (refinementList) {
        var topLevelList = [
          'Categories',
          'Price',
          'Rating',
          'Brand'
        ];

        refinementList.forEach(function (r) {
          if (r.Name && (['Rating', 'ParentCatalogGroup', 'ManufacturerName'].indexOf(r.Name) === -1)) {
            topLevelList.push(r.Name);
          }
        });

        topLevelList = topLevelList.map(function (refinementName) {
          return this.findOrCreateRefinement(refinementName, refinementList);
        }.bind(this));

        return topLevelList;
      },

      refinementsTemplate: function (refinement, name) {
        var multiSelect = refinement && refinement.multiSelect;

        if (name) {

          return {
            name: name,
            title: name,
            widget: "Refinement",
            subview: "All",
            multiSelect: multiSelect,
            refinements: []
          };
        }

        return {
          "Categories": {
            name: "Categories",
            title: "Categories",
            widget: "Refinement",
            subview: "Categories",
            multiSelect: multiSelect,
            refinements: []
          },
          "Price": {
            name: "Price",
            title: "Price",
            widget: "Refinement",
            subview: "Price",
            refinements:[
              {
                name: "Minimum",
                widget: "Price"
              },
              {
                name: "Maximum",
                widget: "Price"
              }
            ]
          },
          "Customer Rating": {
            name: "Customer rating",
            title: "Rating",
            widget: "Refinement",
            subview: "CustomerRating",
            refinements: [
              {
                name: '5',
                stars: 5,
                count: 159,
                widget: "CustomerRating"
              },
              {
                name: '4',
                stars: 4,
                count: 627,
                widget: "CustomerRating"
              },
              {
                name: '3',
                stars: 3,
                count: 398,
                widget: "CustomerRating"
              },
              {
                name: '2',
                stars: 2,
                count: 27,
                widget: "CustomerRating"
              },
              {
                name: '1',
                stars: 1,
                count: 1,
                widget: "CustomerRating"
              }
            ]
          },
          "Brand": {
            name: "Brand",
            title: "Brand",
            widget: "Refinement",
            subview: "Brand",
            multiSelect: multiSelect,
            refinements: []
          }
        };
      },

      fakeData: function () {
        if (!this.__refinement__) {
          this.__refinement__ = this.makeFakeData();
        }
        return this.__refinement__;
      },

      makeFakeData: function () {
        var refinements = [
          [{
            name: "Categories",
            title: "Categories",
            widget: "Refinement",
            subview: "Categories",
            refinements: [
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Interior paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Tester paints",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Wall and ceiling paints",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Floor paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Radiator paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Surface prep paint",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Wallpaper",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Wallpaper paste",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              },
              {
                name: "Borders",
                widget: "TextWithCount",
                count: Math.round(Math.random() * 100)
              }
            ]
          }],
          [
            {
              name: "Price",
              title: "Price",
              widget: "Refinement",
              subview: "Price",
              refinements: [
                {
                  name: "Minimum",
                  widget: "Price"
                },
                {
                  name: "Maximum",
                  widget: "Price"
                }
              ]
            },
            {
              name: "Customer rating",
              title: "Rating",
              widget: "Refinement",
              subview: "CustomerRating",
              refinements: [
                {
                  stars: 5,
                  count: 159,
                  widget: "CustomerRating"
                },
                {
                  stars: 4,
                  count: 627,
                  widget: "CustomerRating",
                  text: true
                },
                {
                  stars: 3,
                  count: 398,
                  widget: "CustomerRating",
                  text: true
                },
                {
                  stars: 2,
                  count: 27,
                  widget: "CustomerRating",
                  text: true
                },
                {
                  stars: 1,
                  count: 1,
                  widget: "CustomerRating",
                  text: true
                }
              ]
            },
            {
              name: "Brand",
              title: "Brand",
              widget: "Refinement",
              subview: "Brand",
              multiSelect: true,
              refinements: [
                {
                  name: "Bosch",
                  count: 46,
                  widget: "TextWithCount"
                },
                {
                  name: "Little Tikes",
                  count: 38,
                  widget: "TextWithCount"
                },
                {
                  name: "Iceland",
                  count: 28,
                  widget: "TextWithCount"
                },
                {
                  name: "Mastercook",
                  count: 13,
                  widget: "TextWithCount"
                }
              ]
            }
          ]
        ];

        var refinement = new Refinement({
          name: "Refine",
          title: "Refine",
          subview: "Refine",
          refinements: refinements
        });

        return refinement;
      }
    };

    return refinementsHelper;
  });