
/* JavaScript content from js/app/util/requestProvider/ServiceRegistryRequestProvider.js in folder common */
/*global define*/
/*jslint regexp: true*/

define(["dojo/request/registry",
  "dojo/Deferred",
  "dojo/_base/lang",
  "dojo/_base/config",
  "dojo/_base/declare"],
  function (registry, Deferred, lang, config, declare) {

    var Srrp = declare('app/util/requestProvider/ServiceRegistryRequestProvider', [], {

      uriScheme : /^registry:\/\/([^\/]+?)$/,

      // dojoConfig key to inspect for the configuration module name
      configProperty : "ServiceRegistryRequestProvider_configModule",
      traceConfigProperty: "ServiceRegistryRequestProvider_trace",

      matches : function (url) {
        return this.uriScheme.test(url);
      },

      _munge : function _munge(service, url, options) {

        switch (typeof service) {
        case "string":
          // Replace the URL
          return {
            url : service,
            options : options
          };
        case "function":
          // Apply the function and return its result
          return service({
            url : url,
            options : options
          });
        case "object":
          // Return the service object as-is
          return service;
        default:
          // Break
          return undefined;
        }
      },

      handle : function (url, options, returnDeferred) {
        var dfd = new Deferred();

        if (!config[this.configProperty]) {
          dfd.reject('Cannot use ServiceRegistryRequestProvider if unconfigured.');
          return returnDeferred ? dfd : dfd.promise;
        }

        var splitInfo = this.uriScheme.exec(url);
        if (!splitInfo) {
          dfd.reject('Invalid service registry URL, scheme is registry://serviceId');
          return returnDeferred ? dfd : dfd.promise;
        }
        var serviceId = splitInfo[1];

        require([config[this.configProperty]], lang.hitch(this, function (registryConfig) {
          var service = registryConfig[serviceId];

          // Munge the url and options according to the service object
          var mungedRequestParams = this._munge(service, url, options);

          if (!mungedRequestParams) {
            //console.error('Invalid service reference in config module - either not found or invalid.');
            dfd.reject('Invalid service reference in config module - either not found or invalid.');
          } else if (mungedRequestParams.url === url) {
            // Applying the munger had no effect on URL, reject this else we'll be iterating for ever
            //console.error('Result of request munger on URL was identical to parameters, aborting.');
            dfd.reject('Result of request munger on URL was identical to parameters, aborting.');
          } else {
            // Send the munged request back into the request registry
            var newDfd = registry(mungedRequestParams.url, mungedRequestParams.options);
            // Use its results as the original request's results
            newDfd.then(lang.hitch(dfd, "resolve"), lang.hitch(dfd, "reject"), lang.hitch(dfd, "progress"));
            // Hook the original request's "cancel" method to this new deferred's cancel method
            dfd.cancel = lang.hitch(newDfd, "cancel");
          }
        }));

        return returnDeferred ? dfd : dfd.promise;
      }
    });

    // Create a singleton instance
    var instance = new Srrp();
    Srrp.getInstance = function getInstance() {
      return instance;
    };
    // Register it with the registry so it handles requests
    registry.register(lang.hitch(instance, "matches"), lang.hitch(instance, "handle"), true);

    // Return the module definition
    return Srrp;
  });