
/* JavaScript content from js/app/util/requestProvider/WorklightRequestProvider.js in folder common */
/*global define,window,WL*/
/*jslint regexp: true*/

define(["dojo/_base/declare",
  "dojo/request/registry",
  "dojo/Deferred",
  "dojo/_base/lang",
  "dojo/_base/kernel",
  "app/nativeMethods"], function (declare, registry, Deferred, lang, kernel, nativeMethods) {

  var Wrp = declare([], {

    uriScheme : /^(wlinvoke|worklight):\/\/([^\/]+?)\/([^\/]+)\/([^\/]+?)$/,

    matches : function (url) {
      return this.uriScheme.test(url);
    },

    handle : function (url, options, returnDeferred) {
      var dfd = new Deferred();

      // Check if we are in WL land and if not, return an error
      if (!kernel.global.WL || kernel.global.WL.mock) {
        dfd.reject('Cannot use WorklightRequestProvider if not in a Worklight environment!');
        //console.error('Cannot use WorklightRequestProvider if not in a Worklight environment!');
        return returnDeferred ? dfd : dfd.promise;
      }

      // Extract the adapter and procedure name
      var splitInfo = this.uriScheme.exec(url) || this.uriScheme_memorable.exec(url);
      if (!splitInfo) {
        dfd.reject('Invalid Worklight URL, scheme is wlinvoke://adapter/procedure OR worklight://adapter/procedure');
        return returnDeferred ? dfd : dfd.promise;
      }
      var adapter = splitInfo[2], procedure = splitInfo[3], version = splitInfo[4];

      if (window.cordova) {
        // Invoke the procedure on the device
        nativeMethods.exec(
          lang.hitch(dfd, "resolve"),
          lang.hitch(dfd, "reject"),
          'WebProxy',
          'invoke',
          [
            version,
            adapter,
            procedure,
            (options && options.parameters) || []
          ]
        );
      } else {
        Array.prototype.insert = function (index, item) {
          this.splice(index, 0, item);
        };
        options.parameters.insert(0, version);
        // Invoke the procedure
        WL.Client.invokeProcedure({
          adapter : adapter,
          procedure : procedure,
          parameters : (options && options.parameters) || []
        }, {
          onSuccess : function (result) {
            dfd.resolve(result.invocationResult);
          },
          onFailure : lang.hitch(dfd, "reject")
        });
      }

      // Return the result deferred
      return returnDeferred ? dfd : dfd.promise;
    }
  });

  // Create a singleton instance
  var instance = new Wrp();
  Wrp.getInstance = function getInstance() {
    return instance;
  };
  // Register it with the registry so it handles requests
  registry.register(lang.hitch(instance, "matches"), lang.hitch(instance, "handle"), false);

  // Return the module definition
  return Wrp;
});