
/* JavaScript content from js/app/views/Product.js in folder common */

/* JavaScript content from js/app/views/Product.js in folder common */
/*global define*/

define("app/views/Product", [
 "dojo/_base/declare",
 "./_ViewBase",
 "dojo/text!./templates/Product.html",
 "dojo/_base/lang",
 "dojo/dom-construct",
 "dojo/promise/all",
 "app/presenters/ProductPresenter",
 "app/presenters/ContentForProductPresenter",
 "app/presenters/ReviewPresenter",
 "dojo/when",
 "dojo/dom-class",
 "dojo/on",
 "dojo/dom",
 "dojo/window",
 "dojo/query",
 "app/nativeMethods",
 "dojo/_base/array",
 "dojo/dom-style",
 "dojo/dom-geometry",
 "app/widgets/Espot",
 "app/widgets/Reviews",
 "app/widgets/PurchasingOptions",
 "dojo/text!../recommendations.json",
 "app/widgets/RelatedProducts",
 "app/widgets/Gallery",
 "app/widgets/Rating"],
  function (
    declare,
    _ViewBase,
    templateString,
    lang,
    domConstruct,
    all,
    ProductPresenter,
    ContentForProductPresenter,
    ReviewPresenter,
    when,
    domClass,
    on,
    dom,
    win,
    query,
    nativeMethods,
    array,
    domStyle,
    domGeom,
    Espot,
    Reviews,
    PurchasingOptions,
    recommendationsFromFile) {
  return declare("app/views/Product", [_ViewBase], {
    templateString: templateString,
    constructor: function () {
      this.params = arguments.params;
    },
    init: function () {
      var p = ProductPresenter.getProduct(this.params.productId);

      when(p, lang.hitch(this, function (product) {
        this.set('product', product);
        var storeOptions = product.storeOptions[0],
          deliveryOptions = product.deliveryOptions[0],
          currentlyOutOfStock = product.currentlyOutOfStock,
          canCheckStock = product.canCheckStock,
          addToBasket = product.addToBasket,
          messages = '';

        if (canCheckStock === false) {
          this._hideDomNode(this.checkStock);
        }
        if (addToBasket === false) {
          this._hideDomNode(this.addToBasket);
          this._hideDomNode(this.productActions);
        }
        if (currentlyOutOfStock === true) {
              this.purchasingOptions.innerHTML = "<p>Currently out of stock</p>";
        }
        if (deliveryOptions) {
          if (deliveryOptions.message) {
            deliveryOptions.message.forEach(function(deliveryMessage) {
               messages += '<p>' + deliveryMessage + '</p>';
            });
          }
          var purchasingOption = new PurchasingOptions({ option: deliveryOptions.option, icon: deliveryOptions.icon, message: messages });
          purchasingOption.placeAt(this.purchasingOptions, "last");
        }
        if (storeOptions) {
          if (storeOptions.message) {
            messages = '<p>' + storeOptions.message + '</p>';
          }
          var purchasingOption = new PurchasingOptions({ option: storeOptions.option, icon: storeOptions.icon, message: messages });
          purchasingOption.placeAt(this.purchasingOptions, "last");
        }
        if (product.hasWasPrice()) {
          this.set({wasPrice: product.wasPrice});
        } else {
          this._hideDomNode(this.wasPrice);
        }
        if (product.hasCurrentPrice()) {
          this.set({currentPrice: product.currentPrice});
        } else {
          this.price.innerHTML = "Price pending";
        }
        if (product.hasReturnsPolicy()) {
          this.set({returnsPolicy: product.returnsPolicy.value});
        } else {
          this._hideDomNode(this.additionalInformation);
        }
        if (product.hasBulletinText()) {
          this.set({bulletinText: product.bulletinText.Bulletin[0].value});
        }
        if (product.hasAverageRating()) {
          this.set({averageRating: product.averageRating});
          this.set({reviewCount: product.numberOfRatings});
        } else {
          this._hideDomNode(this.rating);
        }
        if (product.hasSavingsMessage()) {
          this.set({savingsMessage: product.savingsMessage[0].value});
        } else {
          this._hideDomNode(this.savingText);
        }
        if (product.hasDescription()) {
          this.set({description: product.description});
        } else {
          this._hideDomNode(this.description);
        }
        if (product.isOnlineExclusive) {
          this.set({onlineExclusive: "Online exclusive"});
        } else {
          this._hideDomNode(this.onlineExclusive);
        }
        if (product.hasNectarPoints()) {
          this.set({nectarPoints: product.nectarPoints});
        } else {
          this._hideDomNode(this.nectarPointsNode);
        }
        this.pageReady(product);
        this.displayReviews(product);
        this.initEspots();
        this.images.displayInGallery(product);
        if (window.cordova) {
          nativeMethods.exec(function(placements) {
            this.showPlacements(placements);
          }.bind(this), function() {
            this.showRelatedProducts(product);
          }.bind(this), 'GetRecommendedProducts', 'start', [product.id]);
        } else {
          var placements = this.loadRecommendationsFromStub();
          this.showPlacements(placements);
        }
        //this.initEspots();
      }));
    },
    initEspots: function () {

      if (this.params.useAdapterForEspots === 'true') {
        this.displayEspots();
      } else {
        this.displayCoreMediaEspots();
      }
    },
    displayCoreMediaEspots: function () {

      var coreMediaEspots = [
        {
          dojoAttachName: 'coreMediaEspotsFrame',
          phoneUrlTemplate: "[DOMAIN]/webapp/wcs/stores/servlet/CMContentView?storeId=10201&langId=110&catalogId=10011&placement=*pdpphonetop&productId=[PRODUCT_ID]",
          tabletUrlTemplate: "[DOMAIN]/webapp/wcs/stores/servlet/CMContentView?storeId=10201&langId=110&catalogId=10011&placement=*pdptablettop&productId=[PRODUCT_ID]"
        },
        {
          dojoAttachName: 'coreMediaEspots2Frame',
          phoneUrlTemplate: "[DOMAIN]/webapp/wcs/stores/servlet/CMContentView?storeId=10201&langId=110&catalogId=10011&placement=*pdpphonebottom&productId=[PRODUCT_ID]",
          tabletUrlTemplate: "[DOMAIN]/webapp/wcs/stores/servlet/CMContentView?storeId=10201&langId=110&catalogId=10011&placement=*pdptabletbottom&productId=[PRODUCT_ID]"
        }
      ];

      var urlTemplateSelector = this.isPhone() ? 'phoneUrlTemplate' : 'tabletUrlTemplate';

      this.listenForCoreMediaFrameMessages();

      coreMediaEspots.forEach(function (espotConfig) {
        var frame = this[espotConfig.dojoAttachName];
        var urlTemplate = espotConfig[urlTemplateSelector];
        var previewToken = this.params.previewToken;


        var url = urlTemplate.replace("[DOMAIN]", this.params.WCSURL).replace("[PRODUCT_ID]", this.params.productId);
        // TODO this is an integration-stub while developing offsite
        //var url = 'http://localhost/homebase/postMessageHeight.html';

        if( previewToken ) {
          url += "&previewToken=" + previewToken;
        }

        /*
      for ( p in this.params ) {

        alert( p )
      }
      */

        frame.src = url;
      }.bind(this));
    },
    listenForCoreMediaFrameMessages: function () {
      window.addEventListener('message', function(e) {
        var frame = 'unknown frame';
        var frames = document.querySelectorAll('iframe')

        for (var i = 0; i < frames.length; i++) {
          var f = frames[i];
          //
          // http://dev.w3.org/html5/postmsg/#web-messaging
          //
          // using the source window to check which
          // frame the message came from should work...
          //
          if (e.source == f.contentWindow) {
            frame = f;
            break;
          }
        }
        this.handleCoreMediaFrameMessage(frame, e.data);

      }.bind(this), false);
    },
    handleCoreMediaFrameMessage: function (frame, message) {
      switch (message.code) {
        case 'ready':
          var height = message.height;
          frame.style.height = height + "px";
          frame.style.display = 'block';
          break;
        case 'fail':
          frame.style.display = 'none';
          break;
        default:
          console.warn('unexpected message code:', message.code);
      }
    },
    showPlacements: function(placements) {
      this.relatedProducts.buildPlacements(placements);
    },
    showRelatedProducts: function(product) {
      if (product.hasAlsoInRange()) {
        this.relatedProducts.setAlsoInRange(product.alsoInRange);
      }
      if (product.hasEssentialExtras()) {
        this.relatedProducts.setEssentialExtras(product.essentialExtras);
      }
      if (product.hasAlternatives()) {
        this.relatedProducts.setAlternatives(product.alternatives);
      }
    },
    _setProductAttr: function (product) {
      this.set({
        id: product.id,
        name: product.name,
        maximumQuantity: product.maximumQuantity
      });
    },
    _setIdAttr: {
      type: 'innerHTML',
      node: 'productId'
    },
    _setNameAttr: {
      type: 'innerHTML',
      node: '_nameNode'
    },
    _setDescriptionAttr: {
      type: 'innerHTML',
      node: 'longDescriptionNode'
    },
    _setReturnsPolicyAttr: function(returnsPolicy) {
      this.additionalInformationNode.innerHTML = '<p>&dagger;' + returnsPolicy + '</p>';
    },
    _setBulletinTextAttr: function(bulletinText) {
      this.additionalInformationNode.innerHTML += '<p>' + bulletinText + '</p>';
    },
    _setMaximumQuantityAttr: function (maximumQuantity) {
      var i;
      if (!maximumQuantity) {
        maximumQuantity = 99;
      }
      for (i = 0; i < maximumQuantity; i++) {
        domConstruct.create("option", { innerHTML: [i + 1], value: [i + 1] }, this.maximumQuantityNode);
      }
    },
    _setWasPriceAttr: function (wasPrice) {
      this.wasPrice.innerHTML = "Was £" + wasPrice;
    },
    _setNectarPointsAttr: function(nectarPoints) {
      this.nectarPointsNode.innerHTML = nectarPoints + " Nectar points with this purchase*";
      if (nectarPoints) {
        domConstruct.place("<p>* A maximum of 1000 points can be collected per transaction</p>", this.additionalInformationNode, "after");
        this._showDomNode(this.additionalInformation);
      }
    },
    _setOnlineExclusiveAttr: {
      type: 'innerHTML',
      node: 'onlineExclusive'
    },
    _setSavingsMessageAttr: {
      type: 'innerHTML',
      node: 'savingText'
    },
    _setCurrentPriceAttr: function(CurrentPrice) {
      this.price.innerHTML = "&pound;" + CurrentPrice;
    },
    _setAverageRatingAttr: function (averageRating) {
      this.ratingTop.setRating(averageRating);
    },
    _setReviewCountAttr: function (reviewCount) {
      this.reviewCountTop.innerHTML = reviewCount;
    },
    _hideDomNode: function (node) {
      domStyle.set(node, 'display', 'none');
    },
    _showDomNode: function (node) {
      domStyle.set(node, 'display', 'block');
    },
    _checkStock: function () {
      nativeMethods.exec(null, null, 'CheckStock', 'start', [this.params.productId]);
    },
    isPhone: function () {
      return win.getBox().w < 601;
    },
    displayEspots: function () {
      var deferreds  = [],
      espotWidgets = [],
      espots       = [];

      if (this.isPhone()) {
        espots = [{
          name   : 'sa_phone_pdp_promotion_recomendation',
          height : 104
        }];
      } else {
        espots = [{
          name   : 'sa_tablet_pdp_promotion_recomendation',
          height : 104
        }];
      }
      espots.forEach(function (espot) {
            var c = ContentForProductPresenter.getContent(espot.name, this.params.productId);
        deferreds.push(c);
        when(c, function (content) {
          if (content.baseMarketingSpotActivityData) {
            var widget = new Espot({espot: content, height: espot.height, env: this.params.WCSURL});
            widget.startup();
            espotWidgets.push(widget);
          }
        }.bind(this));
      }.bind(this));
      all(deferreds).then(function () {
        espotWidgets.forEach(function (espotWidget) {
          this.espotsContainer.appendChild(espotWidget.domNode);
        }.bind(this));
        this.espotsContainer.style.display = 'block';
        this._showDomNode(this.inspiration);
      }.bind(this));
    },
    displayReviews: function (product) {
      var r = ReviewPresenter.getReviews(this.params.productId),
      windowWidth = win.getBox();
      when(r, lang.hitch(this, function (reviews) {
            if (reviews.length > 0) {
          var reviewsWidget = new Reviews({ 'Reviews' : reviews, 'Product' : product }),
              reviewsNode = query("#reviews"),
          review = dom.byId("reviews");
          reviewsWidget.set('data', product);
              reviewsWidget.placeAt(this.reviewContainer, "last");
          reviewsWidget.startup();
          this._showDomNode(this.reviews);
          var reviewNode = query('.review p');

          array.forEach(reviewNode, function (text) {
                var parent = text.parentElement,
            container = parent.parentElement,
            textHeight = domGeom.position(text).h,
            containerHeight = domGeom.position(container).h,
            moreButton = text.childNodes[1];
            if (textHeight > containerHeight) {
                domClass.toggle(parent, "closed");
            domStyle.set(moreButton, 'display', 'inline-block');
            }
          });
          reviewsNode.addClass('hasReviews');

          if (windowWidth.w < 601) {

            on(review, "click", function () {
              nativeMethods.exec(null, null, 'PDPReviews', 'start', [product.id]);
            });
          }

        } else {
          this.reviewContainer.innerHTML = "<p>This product has no reviews</p>";
        }
      }));
        },
    loadRecommendationsFromStub: function () {
      return JSON.parse(recommendationsFromFile);
    },
    _addToBasket: function () {
      var quantity = this.maximumQuantityNode.value;
      nativeMethods.exec(console.trace, console.trace, 'Analytics', 'trackAddToBasket', [this.params.productId, quantity]);
      nativeMethods.exec(console.trace, console.trace, 'AddToBasket', 'addProductToBasket', [this.params.productId, quantity]);
    }
  })
});
