
/* JavaScript content from js/app/views/Refine.js in folder common */
/*global define, navigator, window, history*/

define("app/views/Refine", [
  "dojo/_base/declare",
  "dojo/text!./templates/Refine.html",
  "dojo/text!../refinements.json",
  "dojo/text!../appliedRefinements.json",
  "app/presenters/RefinePresenter",
  "dojo/query",
  "dojo/dom-style",
  "dojo/dom-attr",
  "dojo/dom-class",
  "dojo/dom",
  "dojo/when",
  "dijit/registry",
  "dojo/_base/lang",
  "dojo/_base/array",
  "app/nativeMethods",
  "./_RefineBase",
  "app/models/Selection",
  "app/models/Refinement",
  "app/models/searchOptions"],
  function (declare, templateString, refinementsFromFile, appliedRefinementsFromFile, RefinePresenter,
    query, domStyle, domAttr, domClass, dom, when, registry, lang, array, nativeMethods, _RefineBase, selection, Refinement, searchOptions) {

    return declare("app/views/Refine", [_RefineBase], {
      templateString: templateString,
      loadFromStub: function () {
        var payload = JSON.parse(refinementsFromFile),
          appliedRefinements = JSON.parse(appliedRefinementsFromFile);

        return {
          payload : payload,
          appliedRefinements : appliedRefinements
        };
      },
      stashCategories: function (rawSearchResponse)  {
        var rawRefinements = rawSearchResponse.Search.Options.RefinementList.Refinement;

        for(var i = 0; i < rawRefinements.length; i++) {
          var r = rawRefinements[i];
          if (r && r.Name === 'ParentCatalogGroup') {
            this.__stashedCategories__ = r;
            return;
          }
        }
      },
      unstashCategories: function (rawSearchResponse) {
        var rs = rawSearchResponse.Search.Options.RefinementList.Refinement;

        for(var i = 0; i < rs.length; i++) {
          var r = rs[i];
          if (r && r.Name === 'ParentCatalogGroup') {
            break;
          }
        }
        rs.splice(i, 1, this.__stashedCategories__);
        return rawSearchResponse;
      },
      init: function () {
        this.inherited(arguments);

        this.payloadExists = true;

        var payload = window.cordova ? this.params : this.loadFromStub().payload;
        var appliedRefinements = window.cordova ? this.params.appliedRefinements : this.loadFromStub().appliedRefinements;

        Object.size = function(obj) {
          var size = 0, key;
          for (key in obj) {
            if (obj.hasOwnProperty(key)) size++;
          }
          return size;
        };

        var appliedRefinementsSize = Object.size(appliedRefinements);

        console.log(JSON.stringify(payload));

        console.log(appliedRefinements);

        if (appliedRefinements && appliedRefinementsSize > 0) {
          selection.restore(appliedRefinements);
          console.log(JSON.stringify(selection));
        }

        this.extractInitialParams(payload);
        this.extractSearchResults(payload.response);
        this.rawSearchResponse = payload;
        this.stashCategories(payload.response);

        if (this.initialParams.categoryId) {
          this.loadRefinement({ stashCategories: true, showExtraRefinements: true });
        } else {
          this.loadRefinement({ stashCategories: true });
        }

        selection.on('selection-change', function (refinement, newVal) {
          if (refinement.parent && refinement.parent.isCategories()) {
            this.__categoryChanged__ = true;
          }
          this.updateClearButtonVisiblity();
        }.bind(this));

        this.displayNumberOfResults();
        this.populate();
      },
      beforeActivate: function () {
        this.inherited(arguments);
        this.updateClearButtonVisiblity();
      },
      afterActivate: function () {
        this.doSearch();
      },
      extractInitialParams: function (initialParams) {
        this.initialParams = {};
        this.initialParams.categoryId         = initialParams.categoryId;
        this.initialParams.searchTerm         = initialParams.searchTerm || '';
        this.initialParams.orderBy            = initialParams.searchOptions.orderBy;
        this.initialParams.appliedRefinements = initialParams.appliedRefinements;
        this.initialParams.pageSize           = initialParams.searchOptions.pageSize;
      },
      extractSearchResults: function (searchResponse) {
        this.searchResults = {
          refinementList: searchResponse.Search.Options.RefinementList.Refinement,
          totalResults  : searchResponse.Search.totalResults
        };
      },
      loadRefinement: function (options) {
        this.refinement = Refinement.build(this.searchResults.refinementList, options);
      },
      updateClearButtonVisiblity: function () {
        var clearButton = this.clearButton,
          titleNode = this.titleNode;

        if (clearButton) {
          var display = 'none';
          if (this.refinement.hasSubSelection()) {
            display = 'block';
          }

          domStyle.set(clearButton, "display", display);
        }

        if (titleNode) {
          domClass.remove(titleNode, 'visible notVisible');
          var visibility = 'visible';
          if (this.refinement.hasSubSelection()) {
            visibility = 'notVisible';
          } else {
            visibility = 'visible';
          }
          domClass.add(titleNode, visibility);
        }
      },
      updateSearchOptions: function () {
        this.searchOptions = searchOptions.build(this.refinement, this.initialParams.categoryId);
        this.searchOptions.orderBy = this.initialParams.orderBy;
        this.searchOptions.pageSize = this.initialParams.pageSize;

        if (!this.initialParams.searchTerm) {
          this.searchOptions.categoryId = this.initialParams.categoryId;
        }

      },
      doSearch: function () {
        this.updateSearchOptions();

        var resultsNode = query('.refine-results')[0],
            responseErrorCode;

        if (this.payloadExists === false) {

          this.showSpinner();

          console.log("SEARCH TERM " + this.initialParams.searchTerm);
          console.log("SEARCH OPTIONS " + JSON.stringify(this.searchOptions));

          var p = RefinePresenter.getRefinements(this.initialParams.searchTerm, this.searchOptions);

          domClass.add(this.applyButton, 'invalid');

          when(p, lang.hitch(this, function (searchPayload) {
            domClass.remove(this.applyButton, 'invalid');
            this.rawSearchResponse = searchPayload;
            this.extractSearchResults(searchPayload);
            this.displayNumberOfResults();

            if (this.__categoryChanged__) {
              delete this.__categoryChanged__;
              this.loadRefinement();
              this.populate({animate: true});
            }

            if (this.searchOptions.minPrice === 0 && this.searchOptions.maxPrice === 0) {
              resultsNode.innerHTML = 'There are no products for your refinement';
              domClass.add(this.applyButton, 'invalid', true);
            }
          }), function (error) {
            resultsNode.innerHTML = 'There has been an error. Please try again';
            domClass.add(this.applyButton, 'invalid', true);
            if (error.Error) {
              responseErrorCode = error.Error.Code;
            } else {
              responseErrorCode = error.invocationResult.statusCode;
            }
            if (responseErrorCode === "404") {
              resultsNode.innerHTML = "There are no products for your refinement";
            } else {
              resultsNode.innerHTML = "There has been an error. Please try again";
            }
            domClass.add(this.applyButton, 'invalid', true);
          }.bind(this));
        }
      },
      showSpinner: function () {
        var resultsNode = query('.refine-results')[0];
        resultsNode.innerHTML = '<div class="refine-loader"></div>';
      },
      clearRefinements: function () {
        this.refinement.clear();
        delete this.searchOptions.minPrice;
        delete this.searchOptions.maxPrice;
        delete selection.lookup;
        delete selection.price;
        if (this.searchOptions.searchTerm) {
          delete selection.categoryId;
          delete this.initialParams.categoryId;
        }
        this.searchOptions.facet = [];
        this.payloadExists = false;
        this.doSearch();
      },
      displayNumberOfResults: function () {
        var productsText = " product",
            totalResults = this.searchResults.totalResults;

        if (totalResults === 0 || totalResults > 1) {
          productsText = " products";
        }
        this.productsCount.innerHTML = totalResults.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",") + productsText;
      },
      closeRefinementPanel: function () {
        nativeMethods.exec(console.info.bind(console), console.error, "Refinements", "cancelRefinementPanel", []);
      },
      beforeDeactivate: function () {
        var animatedSection = query('.slideIn')[0];
        if (animatedSection) {
          domClass.remove(animatedSection, 'slideIn');
        }
        this.payloadExists = false;
      },
      applyRefinements: function () {
        if (domClass.contains(this.applyButton, "invalid")) {
          return false;
        }
        var searchTerm = this.initialParams.searchTerm,
            categoryId = this.initialParams.categoryId,
            sortOption = this.initialParams.orderBy,
            pageSize = this.initialParams.pageSize,
            isDefault = true,
            rawSearchResponse;

        if (this.refinement.hasSubSelection()) {
          isDefault = false;
        }

        if (searchTerm) {
          this.searchOptions.searchTerm = searchTerm;
        }
        if (categoryId) {
          this.searchOptions.categoryId = categoryId;
        }
        if (sortOption) {
          this.searchOptions.orderBy = sortOption;
        }
        if (pageSize) {
          this.searchOptions.pageSize = pageSize;
        }
        this.searchOptions.pageNumber = 1;

        selection.set('isDefault', isDefault);

        console.log("SELECTION = " + JSON.stringify(selection.serialize()));

        if (this.initialParams.searchTerm) {
          rawSearchResponse = this.unstashCategories(this.rawSearchResponse.response || this.rawSearchResponse);
        } else {
          rawSearchResponse = this.rawSearchResponse.response;
        }

        console.log("RAW SEARCH RESPONSE = " + JSON.stringify(rawSearchResponse));

        console.log("SEARCH OPTIONS = " + JSON.stringify(this.searchOptions));

        nativeMethods.exec(console.info.bind(console), console.error, "Refinements", "applyRefinements", [{
          response: rawSearchResponse,
          searchOptions: this.searchOptions,
          appliedRefinements: selection.serialize()
        }]);
      }
    });
  });