
/* JavaScript content from js/app/views/Registration.js in folder common */
/*global define*/
/*jslint regexp: true*/

define(["dojo/dom",
  "dojo/_base/declare",
  "dojo/text!./templates/Registration.html",
  "./_StaticViewBase",
  "dojo/dom-attr",
  "dojo/query",
  "dojo/dom-class",
  "dojo/dom-style",
  "dojox/validate/web",
  "app/views/_Authentication",
  "app/nativeMethods",
  "app/errors/mapping"],
  function (dom, declare, templateString, _ViewBase, domAttr, query, domClass, domStyle, validate, _Authentication, nativeMethods, mapping) {

    var isValidPassword = function (password) {
      if (password.match(/^(?=(.*\d){2})(?=(.*[a-zA-Z]){4})[0-9a-zA-Z]{6,20}/)) {
        return true;
      }
      return false;
    };

    var isValidPostcode = function (postCode) {
      if (postCode.match(/^([Gg][Ii][Rr] 0[Aa]{2})|((([A-Za-z][0-9]{1,2})|(([A-Za-z][A-Ha-hJ-Yj-y][0-9]{1,2})|(([AZa-z][0-9][A-Za-z])|([A-Za-z][A-Ha-hJ-Yj-y][0-9]?[A-Za-z])))) ?[0-9][A-Za-z]{2})$/)) {
        return true;
      }
      return false;
    };

    var isText = function (string) {
      if (string.match(/^[a-zA-Z \-']+$/)) {
        return true;
      }
      return false;
    };

    var isValidFirstName = function (string) {
      if (string.match(/^[a-zA-Z0-9-]+$/)) {
        return true;
      }
      return false;
    };

    var profile = {
      required: [
        "title",
        "firstName",
        "lastName",
        "postCode",
        "email",
        "confirmEmail",
        "password",
        "confirmPassword"
      ],
      constraints: {
        email     : [validate.isEmailAddress, false, true],
        password  : isValidPassword,
        postCode  : isValidPostcode,
        firstName : isValidFirstName,
        lastName  : isText
      },
      messages: {
        title : {
          missing : this.missingMessageImpersonal + " valid title",
          field   : "title"
        },
        firstName : {
          missing : this.missingMessage + " first name",
          invalid : "Please enter a valid first name",
          field   : "firstName"
        },
        lastName : {
          missing : this.missingMessage + " surname",
          invalid : "Please enter a valid surname",
          field   : "lastName"
        },
        postCode : {
          missing : this.missingMessage + " postcode",
          invalid : this.missingMessageImpersonal + " valid postcode",
          field   : "postCode"
        },
        email : {
          missing  : this.missingMessage + " email address",
          mismatch : "Your email addresses must match",
          invalid  : this.missingMessageImpersonal + " valid email address",
          field    : "email"
        },
        confirmEmail : {
          missing : "Please confirm your email address",
          mismatch : "Your email addresses must match",
          invalid : this.missingMessageImpersonal + " valid email address",
          field   : "confirmEmail"
        },
        password : {
          missing  : this.missingMessageImpersonal + " password", 
          mismatch : "Passwords do not match",
          invalid  : this.missingMessageImpersonal + " valid password",
          field    : "password"
        },
        confirmPassword : {
          missing : "Please confirm your password",
          mismatch : "Passwords do not match",
          invalid  : this.missingMessageImpersonal + " valid password",
          field   : "confirmPassword"
        }
      },
      confirm: {
        "email": "confirmEmail",
        "confirmEmail": "email",
        "password": "confirmPassword",
        "confirmPassword": "password"
      }
    };

    return declare("app/views/Registration", [_Authentication, _ViewBase], {
      templateString: templateString,
      init: function () {
        this.inherited(arguments);
        this.profile = profile;

        this.addClassToBody("register");
      },
      formFieldFailedValidation: function (results, field) {
        if (field) {
          var name = domAttr.get(field, 'name');
          var invalidMessage = this.profile.messages[name].invalid;
          var mismatchMessage = this.profile.messages[name].mismatch;

          if (name === "title" || name === "postCode" || name === "firstName" || name === "lastName") {
            if (results.isInvalid(name)) {
              domStyle.set(dom.byId("detailTitle"), 'display', 'none');
              this.detailResultNode.innerHTML = invalidMessage;
              domClass.add(field.parentElement, "invalid");
            } else {
              domStyle.set(dom.byId("detailTitle"), 'display', 'block');
              this.detailResultNode.innerHTML = "";
              domClass.remove(field.parentElement, "invalid");
            }
          } else {
            var email = query("#email")[0].value,
              confirmEmail = query("#confirmEmail")[0].value,
              password = query("#password")[0].value,
              confirmPassword = query("#confirmPassword")[0].value;

            if (!results.hasInvalid()) {
              domStyle.set(dom.byId("accountTitle"), 'display', 'block');
              this.accountResultNode.innerHTML = "";
              domClass.remove(field.parentElement, "invalid");
            }

            if ((name === 'password' || name === 'confirmPassword') && email !== confirmEmail) {
              domStyle.set(dom.byId("accountTitle"), 'display', 'none');
              this.accountResultNode.innerHTML = 'Your email addresses must match';
            }

            if (results.isInvalid(name) && name !== 'confirmPassword') {
              domStyle.set(dom.byId("accountTitle"), 'display', 'none');
              this.accountResultNode.innerHTML = invalidMessage;
              domClass.add(field.parentElement, "invalid");
            } else if (name === 'confirmEmail' && email !== confirmEmail && confirmEmail !== "") {
              domStyle.set(dom.byId("accountTitle"), 'display', 'none');
              this.accountResultNode.innerHTML = mismatchMessage;
              domClass.add(field.parentElement, "invalid");
            } else if (name === 'confirmPassword' && password !== confirmPassword && confirmPassword !== "") {
              domStyle.set(dom.byId("accountTitle"), 'display', 'none');
              this.accountResultNode.innerHTML = mismatchMessage;
              domClass.add(field.parentElement, "invalid");
            }

          }

        }
      },
      validateField: function (e) {
        this.inherited(arguments);
        var formValidation = validate.check(this.form, this.profile),
          field = e.currentTarget;

        if (!formValidation.isSuccessful()) {
          this.formFieldFailedValidation(formValidation, field);
        }
      },
      formFailedValidation: function (results) {
        this.inherited(arguments);
		
        nativeMethods.exec(console.trace, console.trace, 'Analytics', 'trackValidationError', ['Registration', this.coreMetricsMessage(results)]);
      },
      formPassedValidation: function (formData) {
        this.inherited(arguments);

        nativeMethods.exec(function() { this.showSuccess(); }.bind(this), function(error) { this.showErrors(error); }.bind(this), 'NativeChallengeHandler', 'register', [ formData ]);
      },
      showErrors: function (error) {
        var serverErrors = mapping.errors(error),
          errorFields = serverErrors.fields;
        window.scrollTo(0, 0);
        this.detailResultNode.innerHTML = serverErrors.message;
        errorFields.forEach(function(field) {
          var errorField = dom.byId(field);
          domClass.add(errorField.parentElement, "invalid");
        });
      },
      showSuccess: function () {
        window.nativeNavigate('Success');
      },
      setTitle: function (e) {
        var target = e.currentTarget;

        if (target.value !== "") {
          domStyle.set(target.previousElementSibling, "display", "none");
        } else {
          domStyle.set(target.previousElementSibling, "display", "block");
        }
      }
    });
  });