
/* JavaScript content from js/app/views/Reviews.js in folder common */
/*global define,event*/

define("app/views/Reviews", [
  "dojo/_base/declare",
  "./_ViewBase",
  "dojo/text!./templates/Reviews.html",
  "dojo/_base/lang",
  "dojo/query",
  "app/presenters/ProductPresenter",
  "dojo/when",
  "app/widgets/ProductReview",
  "dojo/_base/array",
  "dojo/window",
  "dojo/on",
  "fastclick/fastclick",
  "dojo/dom-construct",
  "dgrid/OnDemandList",
  "app/stores/ReviewStoreRest",
  "app/widgets/ReviewsContainer",
  "app/widgets/Rating",
  "dojo/NodeList-dom"],
  function (declare, _ViewBase, templateString, lang, query, ProductPresenter, when, ProductReview, array, win, on, FastClick, domConstruct, OnDemandList, ReviewStoreRest, ReviewsContainer) {
    return declare("app/views/Reviews", [_ViewBase], {
      templateString: templateString,
      init: function () {
        var r = ProductPresenter.getProduct(this.params.productId);

        when(r, lang.hitch(this, function (result) {
          this.productData = result;
          this.setupReviewsStore();
          this.setUpOnDemandList();
          this._showFirstMostHelpful();
          this.pageReady();
        }));
      },
      setupReviewsStore: function () {
        this.reviewStore = new ReviewStoreRest({
          idProperty: "id",
          params: {
            id: this.params.productId,
            resultsPerPage: 10,
            sort: "Helpfulness:desc,TotalPositiveFeedbackCount:desc,SubmissionTime:desc"
          }
        });
      },
      setUpOnDemandList: function () {
        if (this.reviewList === undefined) {
          this.reviewList = new OnDemandList({
            store: this.reviewStore,
            renderRow: this.formatRow,
            pagingDelay: 0,
            bufferRows: 0,
            keepScrollPosition: false,
            farOffRemoval: 0,
            loadingMessage: "<div class='loader'>Loading...</div>"
          }, this.reviewsGrid);

          this.reviewList.startup();
        } else {
          this.reviewList.refresh();
        }
      },
      _showFiltered: function (sortBy, initialLoad) {
        this.reviewStore.params.sort = sortBy;
        this.reviewList.startup();
        if (!initialLoad) {
          this.reviewList.refresh();
        }

        var windowDimensions = win.getBox(),
          dgridScroller      = query(".dgrid-scroller"),
          dgridContent       = query(".dgrid-content")[0],
          tempDiv            = domConstruct.create("div", null, dgridContent, "first"),
          reviewsContainer   = new ReviewsContainer().placeAt(tempDiv);

        reviewsContainer.set('data', this.productData);

        dgridScroller.style("height", windowDimensions.h + "px");

        on(reviewsContainer.highestRating, "click", lang.hitch(this, function () {
          this._showHighestRating(event);
        }));

        on(reviewsContainer.mostRecent, "click", lang.hitch(this, function () {
          this._showMostRecent(event);
        }));

        on(reviewsContainer.mostHelpful, "click", lang.hitch(this, function () {
          this._showMostHelpful(event);
        }));

        var fastclick = this.domNode.querySelectorAll('a');

        array.forEach(fastclick, function (fc) {
          var fclick = new FastClick(fc);
          return fclick;
        });

      },
      _showFirstMostHelpful: function() {
        this._showFiltered("Helpfulness:desc,TotalPositiveFeedbackCount:desc,SubmissionTime:desc", "initialLoad");
        query('.mostHelpful').addClass('selected');
      },
      _showMostHelpful: function (event) {
        this._showFiltered("Helpfulness:desc,TotalPositiveFeedbackCount:desc,SubmissionTime:desc");
        query('.mostHelpful').addClass('selected');
        this.preventDefaultActionOnTabs(event);
      },
      _showMostRecent: function (event) {
        this._showFiltered("SubmissionTime:desc");
        query('.mostRecent').addClass('selected');
        this.preventDefaultActionOnTabs(event);
      },
      _showHighestRating: function (event) {
        this._showFiltered("Rating:desc");
        query('.highestRating').addClass('selected');
        this.preventDefaultActionOnTabs(event);
      },
      formatRow: function (review) {
        var _ri = new ProductReview();
        _ri.set('data', review);
        _ri.startup();
        return _ri.domNode;
      },
      preventDefaultActionOnTabs: function (event) {
        if (event) {
          event.preventDefault();
        }
      }
    });
  });