
/* JavaScript content from js/app/views/_Authentication.js in folder common */
/*global define,navigator*/

define(["dojo/_base/declare",
  "./_ViewBase",
  "dojo/dom",
  "dojo/query",
  "dojo/dom-style",
  "dojo/dom-class",
  "dojo/_base/event",
  "dojo/dom-form",
  "dojo/_base/array",
  "dojo/_base/lang",
  "dojo/dom-attr",
  "dojox/validate/web",
  "dojox/validate/check",
  "dojo/NodeList-dom"],
  function (declare, _ViewBase, dom, query, domStyle, domClass, baseEvent, domForm, array, lang, domAttr, validate) {

    this.confirmMessage = "Please confirm your";
    this.missingMessage = "Please enter your";
    this.missingMessageImpersonal = "Please enter a";

    return declare("app/views/Authentication", [_ViewBase], {
      init: function () {
        this.form = query("form")[0];

        this.detailResultNode = dom.byId("detailResult");
        this.accountResultNode = dom.byId("accountResult");

        this.addClassToBody("authentication");

        if (navigator.userAgent.match(/Android/i)) {
          this.addClassToBody("android");
        }

        if (navigator.userAgent.match(/(iPad)|(iPhone)/i)) {
          this.addClassToBody("ios");
        }
      },
      addClassToBody: function (bodyClass) {
        query("body").addClass(bodyClass);
      },
      emailValidation: function (formValidation) {
        var emailFields = query('input[type="email"]');

        array.forEach(emailFields, function(e) {
          if (e.value && e.value === "" || dom.byId("confirmEmail") && dom.byId("confirmEmail").value !== e.value) {
            formValidation.getInvalid().push(e.name);
          }
        });
      },
      notifyValidator: function (e) {
        var submitButton = dom.byId("submit"),
          formValidation = validate.check(this.form, this.profile);

        if (e.currentTarget.value) {
          domClass.remove(e.currentTarget.parentElement, "invalid");
        }

        this.emailValidation(formValidation);

        if (formValidation.isSuccessful()) {
          domClass.remove(submitButton, 'invalid');
        } else {
          domClass.add(submitButton, 'invalid');
        }
      },
      validateField: function () {
        var submitButton = dom.byId("submit"),
          formValidation = validate.check(this.form, this.profile);

        this.emailValidation(formValidation);

        if (formValidation.isSuccessful()) {
          domClass.remove(submitButton, 'invalid');
        } else {
          domClass.add(submitButton, 'invalid');
        }
      },
      showResetIcon: function (e) {
        var clearIcon = query(e.currentTarget.nextElementSibling);
        if (e.currentTarget.value) {
          clearIcon.style('display', 'block');
        } else {
          clearIcon.style('display', 'none');
        }
      },
      resetField: function (e) {
        var clearIcon = query(e.currentTarget),
          field = query(e.currentTarget.previousElementSibling);

        domAttr.set(field[0], 'value', '');
        clearIcon.style('display', 'none');
        var submitButton = dom.byId("submit");
        domClass.add(submitButton, 'invalid');
      },
      submit: function (e) {
        var formValidation = validate.check(this.form, this.profile);
        var formData = domForm.toObject(this.form);

        baseEvent.stop(e);

        this.emailValidation(formValidation);

        if (formValidation.isSuccessful()) {
          this.formPassedValidation(formData);
        } else {
          this.formFailedValidation(formValidation);
        }
      },
      createPlaceholder: function (node, message) {
        query(node).map(lang.hitch(this, function (node) {
          domClass.add(node.parentElement, "invalid");
          domAttr.set(node, 'placeholder', message);
        }));
      },
      coreMetricsMessage: function(validationResults) {
      var missingFields = validationResults.getMissing();
    if (missingFields.length > 0) {
      var firstMissingField = missingFields[0]; // we only need the first missing field
      return this.profile.messages[firstMissingField].missing;
    }
    return "";
      },
      formFailedValidation: function (results) {
        var missing = results.getMissing(),
          emailFields = query('input[type="email"]'),
          message,
          field,
          node,
          i;

        array.forEach(emailFields, function(e) {
          if (e.value === "") {
            missing.push(e.name);
          }
        });

        for (i in missing) {
          if (missing.hasOwnProperty(i)) {
            field = this.profile.messages[missing[i]];
            node = dom.byId(field.field);
            message = field.missing;

            this.createPlaceholder(node, message);

            if (field.field === "title") {
              this.detailResultNode.innerHTML = message;
              domStyle.set(dom.byId("detailTitle"), 'display', 'none');
            }
          }
        }
      }
    });
  });