
/* JavaScript content from js/app/widgets/Gallery.js in folder common */
/*global define,window,bullets*/

define([
  "dojo/_base/declare",
  "dijit/_WidgetBase",
  "dijit/_TemplatedMixin",
  "dojo/text!./templates/Gallery.html",
  "dojo/_base/array",
  "dojo/dom-construct",
  "dojo/query",
  "dojo/dom-style",
  "dojo/dom-attr",
  "swipe/swipe",
  "app/nativeMethods",
  "app/widgets/_ImageButton",
  "fastclick/fastclick",
  "dojo/window",
  "dojo/NodeList-dom"],
  function (declare, _WidgetBase, _TemplatedMixin, templateString, array, domConstruct, query, domStyle, domAttr, Swipe, nativeMethods, ImageButton, FastClick, win) {
    return declare("app/widgets/Gallery", [_WidgetBase, _TemplatedMixin], {
      templateString: templateString,
      displayInGallery: function (product) {
        var ul = domConstruct.create("ul", null, "nav", "first"),
          windowWidth = win.getBox().w,
          imageSize;

        if (windowWidth < 600) {
          imageSize = 640;
        } else {
          imageSize = 900;
        }

        var scroller = this.scroller,
          productImages = product.images,
          imageCount = 0,
          videoCount = 0,
          spinSetCount = 0;

        var images = (array.filter(productImages, function (pi) {
          return pi.usage === "image";
        }));

        var videos = (array.filter(productImages, function (pi) {
          return pi.usage === "video";
        }));

        var spinSet = (array.filter(productImages, function (pi) {
          return pi.usage === "spinSet";
        }));

        array.forEach(images, function(image) {
          imageCount++;
          var imageButton = new ImageButton(image);
          imageButton.set('imageUrl', image.href + "?wid=" + imageSize + "&hei=" + imageSize);
          imageButton.set('ariaLabel', product.name + ' - image ' + imageCount);
          imageButton.placeAt(scroller, "last");
          domStyle.set(scroller, 'display', 'block');
          imageButton.startup();
          domConstruct.create("li", { innerHTML: "<span class='dot'></span>" }, ul);
        });

        array.forEach(videos, function(video) {
          if (video.type && video.type === "video/mp4") {
            videoCount++;
            var time = '',
              div = domConstruct.create("div", null, "images", "last");

            if (video.Size) {
              var seconds = video.Size.seconds,
                date = new Date(seconds * 1000),
                mm = date.getUTCMinutes(),
                ss = date.getSeconds();
              if (ss < 10) { ss = "0" + ss; }
              time = "<h4 class='time'>" + mm + ":" + ss + "</h4>";
            }
            domConstruct.create("div", { "class": "media", innerHTML: "<i class='icon-video-large'></i><h3>" + video.Description + "</h3>" + time + "<h4 class='instruction'>Tap to play<h4>" }, div);
            domConstruct.create("li", { innerHTML: "<i class='icon-video'></i>" }, ul);
          }
        });

        array.forEach(spinSet, function(spin) {
          spinSetCount++;
          var element = domConstruct.create("div", null, "images", "last");
          domConstruct.create("div", { "class": "media", innerHTML: "<i class='icon-spin-large'></i><h3>360<sup>&deg;</sup> view</h3><h4 class='instruction'>Tap to start</h4>" }, element);
          domConstruct.create("li", { innerHTML: "<i class='icon-spin'></i>" }, ul);
        });

        var totalNumberOfGalleryItems = imageCount + videoCount + spinSetCount,
          nav = query("#nav");

        if (totalNumberOfGalleryItems === 1) {
          domStyle.set(nav[0], 'display', 'none');
        }

        query(".swipe li:first-child").addClass("on");

        var swiper = this.domNode;

        document.swipe = new Swipe(swiper, {
          callback: function (pos) {
            var i = bullets.length;
            while (i--) {
              bullets[i].className = ' ';
            }
            bullets[pos].className = 'on';
          }
        });

        document.reinit = function (startslide) {
          document.swipe = new Swipe(swiper, {
            startSlide: startslide,
            callback: function (pos) {
              var i = bullets.length;
              while (i--) {
                bullets[i].className = ' ';
              }
              bullets[pos].className = 'on';
            }
          });
        };

        window.addEventListener("orientationchange", function () {
          var startslide = document.swipe.getPos();
          document.swipe.kill();
          document.reinit(startslide);
        }, false);

        var bullets = query('.swipe li');
        var images = query('.swipe > div > div');

        var fastclick = bullets.concat(images);

        array.forEach(fastclick, function (fc) {
          var fclick = new FastClick(fc);
          return fclick;
        });

        bullets.forEach(function (event, i) {
          var bullet = query(event);
          bullet.on("click", function () {
            document.swipe.slide(i);
          });
        });

        query('.swipe > div > div').on("click", function (e) {
          nativeMethods.exec(null, null, 'InAppGallery', 'start', [{
            content: product.images,
            productName: product.name,
            index: domAttr.get(e.currentTarget, 'data-index')
          }]);
        });
      }
    });
  });