
/* JavaScript content from js/app/widgets/Product.js in folder common */
/*global define*/

define([
  "dojo/_base/declare",
  "dijit/_WidgetBase",
  "dojo/_base/array",
  "dijit/_TemplatedMixin",
  "dijit/_WidgetsInTemplateMixin",
  "app/widgets/_Product",
  "dojo/text!./templates/Product.html",
  "app/nativeMethods",
  "dojo/dom-style",
  "app/widgets/_ImageButton"],
  function (declare, _WidgetBase, array, _TemplatedMixin, _WidgetsInTemplateMixin, _Product, templateString, nativeMethods, domStyle, ImageButton) {
    return declare("app/widgets/Product", [_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _Product], {
      templateString: templateString,
      extractImage: function (wlProduct) {
        var opts = wlProduct.AssociatedMedia.Content;

        var image = (array.filter(opts, function (opt) {
          return opt.usage === 'image';
        })[0] || {value: ""}).href;

        return {
          image : image
        };
      },
      extractPrice: function (wlProduct) {

        if (wlProduct.PricingInformation) {

          var opts = wlProduct.PricingInformation.Price;

          var was = (array.filter(opts, function (opt) {
            return opt.type === 'was';
          })[0] || {value: ""}).Amount;

          var current = (array.filter(opts, function (opt) {
            return opt.type === 'current';
          })[0] || {value: ""}).Amount;

          var savingsMessage = (array.filter(opts, function (opt) {
            return opt.type === 'current';
          })[0] || {value: ""});

          return {
            was: was,
            current: current,
            savingsMessage: savingsMessage
          };

        } else {
          return false;
        }
      },
      extractRatingInformation: function (wlProduct) {
        var opts = wlProduct.Rating;

        var averageRating = opts.AverageRating;

        var numberOfRatings = opts.NumberReviews;

        return {
          averageRating: averageRating,
          numberOfRatings: numberOfRatings
        };
      },
      _setDataAttr : function (_data) {
        this.inherited(arguments);
        if (!this._data.id) {
          domStyle.set(this.domNode, 'display', 'none');
        } else {
          var description = this._data.name || "";
          var image = this.extractImage(this._data).image;

          var price = this.extractPrice(this._data).current;
          var wasPrice = this.extractPrice(this._data).was;
          var savingsMessage = this.extractPrice(this._data).savingsMessage;
          var rating = this.extractRatingInformation(this._data);

          if (description) {
            this.set("Description", description);
          } else {
            domStyle.set(this._desc, 'display', 'none');
          }

          if (image) {
            this.set("Thumbnail", image);
          } else {
            domStyle.set(this._image, 'display', 'none');
          }

          if (price) {
            if (price.value.toString().indexOf(".") === -1) {
              price.value += ".00";
            }
            price.value = (price.value*1).toFixed(2);
            this.set("Price", price.value);
          } else {
            this._price.innerHTML = "Price pending";
          }

          if (wasPrice && wasPrice !== null && wasPrice.value !== null && wasPrice.value*1 !== 0) {
            if (wasPrice.value.toString().indexOf(".") === -1) {
              wasPrice.value += ".00";
            }
            wasPrice.value = (wasPrice.value*1).toFixed(2);
            this.set("wasPrice", wasPrice.value);
          } else {
            domStyle.set(this._wasPrice, 'display', 'none');
          }

          if (savingsMessage && savingsMessage.Commentary && savingsMessage.Commentary.length && typeof savingsMessage.Commentary[0].value === "string") {
            this.set("savingsMessage", savingsMessage.Commentary[0].value);
          } else {
            domStyle.set(this._savingText, 'display', 'none');
          }

          if (rating.averageRating && rating.averageRating) {
            this.set("AverageRating", rating.averageRating);
            this.set("ReviewCount", rating.numberOfRatings);
          } else {
            domStyle.set(this._rating, 'display', 'none');
          }
        }
      }
    });
  });