
/* JavaScript content from js/app/widgets/SlickImage.js in folder common */
/*global define */

define(["dojo/_base/declare",
  "dijit/_WidgetBase",
  "dijit/_TemplatedMixin",
  "dojo/text!./templates/SlickImage.html",
  "dojo/_base/lang",
  "dojo/Deferred",
  "dojo/dom-construct",
  "dojo/dom-attr",
  "../controllers/BusyController",
  "dojox/css3/transit"],
  function (declare, _WidgetBase, _TemplatedMixin, templateString, lang, Deferred, domConstruct, domAttr, BusyController, transit) {
    return declare("app/widgets/SlickImage", [_WidgetBase, _TemplatedMixin], {
      templateString: templateString,
      _setAriaLabelAttr: function (ariaLabel) {
        this._set('ariaLabel', ariaLabel);
      },
      _setImageUrlAttr: function (imageUrl) {
        this._set('imageUrl', imageUrl);
        var d = new Deferred();

        // Pre-load the image
        var img = domConstruct.create('img');
        img.onload = function () {
          d.resolve();
        };
        img.onerror = function (e) {
          console.error(e);
          d.reject();
        };
        img.src = imageUrl;

        // When the image is preloaded, show it
        d.then(lang.hitch(this, "_showImage", imageUrl));

        // Show the busy indicator if it takes more than 50ms to load the image
        // This should effectively mean that it shows if the image isn't cached
        setTimeout(lang.hitch(this, function () {
          if (!d.isFulfilled()) {
            this._showBusy();
          }
        }), 50);
      },
      _showImage: function (imageUrl) {
        this.image.style.backgroundImage = "url('" + imageUrl + "')";
        domAttr.set(this.image, 'aria-label', this.ariaLabel);
        this._show(this.image);
      },
      _showBusy: function () {
        if (!this.busy) {
          this.busy = BusyController.create(this.domNode, "", this.busySize, { style: { display: 'none' } });
          BusyController.show(this.busy);
        }
        this._show(this.busy, true);
      },
      _show: function (node, fade) {
        if (this._showing || fade) {
          // Fade in
          transit(this._showing, node, { transition: "fade" });
        } else {
          // Don't fade in if nothing was showing before - just show it
          node.style.display = "";
        }
        this._showing = node;
      }
    });
  });